/*
	This file is part of lobj.

	lobj is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	lobj is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with lobj.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2008 Lucas Hermann Negri
*/

#include "date.h"

static int priv_date_gc(lua_State* L)
{
	#ifdef IDEBUG
	g_debug("Garbage collecting a GDate!");
	#endif
	
	Object* obj = lua_touserdata(L, 1);
	g_date_free((GDate*)obj->pointer);
	
	return 0;
}

int lobj_date_new(lua_State* L)
{
	Object* obj = lua_newuserdata(L, sizeof(Object));
	obj->pointer = g_date_new();
	obj->need_unref = TRUE; /* Not used, just for consistency */
	int top = lua_gettop(L);
	
	/* Shared metatable */
	if(luaL_newmetatable(L, "__lobjDaTeMt"))
	{
		lua_getfield(L, LUA_GLOBALSINDEX, "__lobjRegisterSpecial");
		lua_pushvalue(L, -2);
		lua_pushliteral(L, "lobj");
		lua_pushliteral(L, "Date");
		lua_pushcfunction(L, priv_date_gc);
		lua_call(L, 4, 0);
	}

	/* Set the object metatable */
	lua_setmetatable(L, top);
	
	return 1;
}

int lobj_date_set(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	luaL_checktype(L, 2, LUA_TNUMBER);
	luaL_checktype(L, 3, LUA_TNUMBER);
	luaL_checktype(L, 4, LUA_TNUMBER);
	
	Object* obj = lua_touserdata(L, 1);
	g_date_set_dmy((GDate*)obj->pointer, lua_tointeger(L, 2), 
		lua_tointeger(L, 3), lua_tointeger(L, 4));
		
	return 0;
}

int lobj_date_get(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	const GDate* date = (GDate*)((Object*)lua_touserdata(L, 1))->pointer;
	
	lua_pushinteger(L, g_date_get_day(date));
	lua_pushinteger(L, g_date_get_month(date));
	lua_pushinteger(L, g_date_get_year(date));
	
	return 3;
}

int lobj_date_valid(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TNUMBER);
	luaL_checktype(L, 2, LUA_TNUMBER);
	luaL_checktype(L, 3, LUA_TNUMBER);
	
	lua_pushboolean(L, g_date_valid_dmy(lua_tointeger(L, 1), 
		lua_tointeger(L, 2), lua_tointeger(L, 3)));
		
	return 1;
}

int lobj_date_compare(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);	
	luaL_checktype(L, 2, LUA_TUSERDATA);
	
	const GDate* date1 = (GDate*)((Object*)lua_touserdata(L, 1))->pointer;
	const GDate* date2 = (GDate*)((Object*)lua_touserdata(L, 2))->pointer;
	lua_pushinteger(L, g_date_compare(date1, date2));
	
	return 1;
}
