/*
	This file is part of lobj.

	lobj is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	lobj is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with lobj.  If not, see <http://www.gnu.org/licenses/>.

    Copyright (C) 2008 Lucas Hermann Negri
*/

#include "main.h"

static int priv_main_gc(lua_State* L)
{
	#ifdef IDEBUG
	g_debug("Garbage collecting a GMainLoop!");
	#endif

	Object* obj = lua_touserdata(L, 1);
	g_main_loop_unref((GMainLoop*)obj->pointer);

	return 0;
}

static int priv_main_tostring(lua_State* L)
{
	Object* obj = lua_touserdata(L, 1);

	char name[31];
	snprintf(name, 30, "GMainLoop: %p", obj->pointer);
	lua_pushstring(L, name);

	return 1;
}

int lobj_main_loop_new(lua_State* L)
{
	Object* ptr = lua_newuserdata(L, sizeof(Object));
	ptr->pointer = g_main_loop_new(NULL, FALSE);

	#ifdef IDEBUG
	g_debug("Creating a new GMainLoop!");
	#endif

	/* Set the metatable */
	if(luaL_newmetatable(L, "lobj.mainloop"))
	{
		/* A new metatable */
		#ifdef IDEBUG
		g_debug("Creating a new metatable!");
		#endif

		/* __gc */
		lua_pushliteral(L, "__gc");
		lua_pushcfunction(L, priv_main_gc);
		lua_rawset(L, -3);

		/* __tostring */
		lua_pushliteral(L, "__tostring");
		lua_pushcfunction(L, priv_main_tostring);
		lua_rawset(L, -3);

		/* __eq */
		lua_pushliteral(L, "__eq");
		lua_pushcfunction(L, lobj_object_eq);
		lua_rawset(L, -3);

		/* __index */
		lua_pushliteral(L, "__index");
		lua_pushliteral(L, "__lobjMainLoop");
		lua_rawget(L, LUA_GLOBALSINDEX);
		lua_rawset(L, -3);
	}

	/* Set the metatable */
    lua_setmetatable(L, -2);

	return 1;
}

int lobj_main_loop_run(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* ptr = lua_touserdata(L, 1);

	g_main_loop_run((GMainLoop*)ptr->pointer);

	return 0;
}

int lobj_main_loop_iterate(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* ptr = lua_touserdata(L, 1);
	int top = lua_gettop(L);
	gboolean block = TRUE;

	if(top > 1)
	{
		block = lua_toboolean(L, 2);
	}

	gboolean res = g_main_context_iteration(g_main_loop_get_context((GMainLoop*)ptr->pointer), block);
	lua_pushboolean(L, res);

	return 1;
}

int lobj_main_loop_pending(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* ptr = lua_touserdata(L, 1);

	lua_pushboolean(L, g_main_context_pending(g_main_loop_get_context((GMainLoop*)ptr->pointer)));
	return 1;
}

int lobj_main_loop_quit(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* ptr = lua_touserdata(L, 1);

	g_main_loop_quit((GMainLoop*)ptr->pointer);

	return 0;
}
