/*
	This file is part of lobj.

	lobj is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	lobj is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with lobj.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2008 Lucas Hermann Negri
*/

#include "timer.h"

static int priv_timer_gc(lua_State* L)
{
	#ifdef IDEBUG
	g_debug("Garbage Collecting a GTimer!");
	#endif
	
	Object* obj = lua_touserdata(L, 1);	
	g_timer_destroy((GTimer*)obj->pointer);
	
	return 1;
}

int lobj_timer_new(lua_State* L)
{
	#ifdef IDEBUG
	g_debug("Constructing a new GTimer!");
	#endif
	
	Object* obj = (Object*)lua_newuserdata(L, sizeof(Object));
	obj->pointer = g_timer_new();
	obj->need_unref = TRUE; /* Not used, just for consistency */
	int top = lua_gettop(L);
	
	/* Shared metatable */
	if(luaL_newmetatable(L, "__lobjTiMeR"))
	{
		lua_getfield(L, LUA_GLOBALSINDEX, "__lobjRegisterSpecial");
		lua_pushvalue(L, -2);
		lua_pushliteral(L, "lobj");
		lua_pushliteral(L, "Timer");
		lua_pushcfunction(L, priv_timer_gc);
		lua_call(L, 4, 0);
	}

	/* Set the object metatable */
	lua_setmetatable(L, top);
		
	return 1;
}

int lobj_timer_start(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	
	g_timer_start((GTimer*)obj->pointer);
	
	return 0;
}

int lobj_timer_stop(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	
	g_timer_stop((GTimer*)obj->pointer);
	
	return 0;
}

int lobj_timer_continue(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	
	g_timer_continue((GTimer*)obj->pointer);
	
	return 0;
}

int lobj_timer_elapsed(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TUSERDATA);
	Object* obj = lua_touserdata(L, 1);
	
	gulong micro;
	gdouble sec = g_timer_elapsed((GTimer*)obj->pointer, &micro);
	
	lua_pushnumber(L, sec);
	lua_pushnumber(L, micro);
	
	return 2;
}
