/*
	This file is part of lobj.

	lobj is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	lobj is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with lobj.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2008 Lucas Hermann Negri
*/

#ifndef LOBJ_TYPES
#define LOBJ_TYPES

/**
 * GTypes for internal use. Helps table convertions.
 */
typedef enum
{
	TYPE_STRING, TYPE_DOUBLE, TYPE_BOOLEAN, TYPE_ENUM,
	TYPE_STRUCT, TYPE_OBJECT, TYPE_CUSTOM
} Type;

/**
 * A object representation. Can hold a simple struct or a GObject.
 * Retains a pointer to the real object, and if it needs an [unref or free]
 * on GC.
 */
typedef struct
{
	gpointer pointer;
	gboolean need_unref;
} Object;


/**
 * Boxed value. Some GBoxed needs to be around until its used, and this
 * structure is used to store information to free then on GC.
 */
typedef struct
{
	gpointer pointer;
	GType type;
} Boxed;

/**
 * Generic info for the handler.
 */
typedef struct
{
	const GType* param_types;
	GType return_type;
	guint n_params;
} SignalInfo;

/**
 * Callback data.
 * Contains the lua_State to use and the reference to the function to call,
 * and all the info that the handler must use.
 */
typedef struct
{
	lua_State* L;
	int function_ref;
	int ud_ref;
	SignalInfo signal_info;
} Data;

/**
 * For type registering.
 */
struct RType
{
	const gchar* name;
	GType value;
};

#endif
